#region File Description
//-----------------------------------------------------------------------------
// GameplayScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Diagnostics;
using System.Threading;
using System.Text;
#endregion

namespace AcidRainTutorialNew
{
   
    public class GameplayScreen : GameScreen
    {
        ContentManager content;
        public static Vector2 PlayerStartPos = new Vector2(1500, 800); 

        #region GamePlay Controls
        PlayerIndex playerIndex;
        InputState input;
        KeyboardState keyboardState;
        GamePadState gamePadState;
        public static bool ComponentsMainMapPauseResume;    //Set to true in screenManager remove screen method when resuming from pause to restart components
        public static bool ComponentsCaveMapPauseResume;    //Set to true in screenManager remove screen method when resuming from pause to restart components
        #endregion

        #region Game Component declarations
        Terrain terrain;
        FrontOfPlayer frontOfPlayer;
        Player player;
        #endregion

        public GameplayScreen()
        {
            TransitionOnTime = TimeSpan.FromSeconds(1.5);
            TransitionOffTime = TimeSpan.FromSeconds(0.5);
        }

        public override void LoadContent()
        {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

  
            #region Set Game Switch Defaults
            PlayerStartPos = new Vector2(1500, 800);
            ComponentsMainMapPauseResume = false;
            ComponentsCaveMapPauseResume = false;
            ScreenManager.Camera.scale = 1f; //Reset camera to starting scale
            ScreenManager.Camera.Position = new Vector2(0, 0);
            #endregion

            //Game Services is a way for Game Components to communicate with each other since each component has a copy of Game
            //Add to content from GamePlayScreen to Game Services
            //I can then access this contentmanager from any gameplay class or Component and 
            //it will be unloaded when I call content.unload in this class.
            ScreenManager.Game.Services.AddService(typeof(ContentManager), content);

            // Look up inputs for the active player profile.
            //Instead of declaring this here as in the example, declare at top of class for use throughout class
            playerIndex = ControllingPlayer.Value;
             
            #region Instantiate Game Component Classes and determine draworder
            //Game components have their own Update and Draw methods.  This allows me to 
            //use a different spriteBatch.Begin in each Component Draw method.  In HUD, I am able to avoid the camera by not calling 
            //camera.transfrom in the HUD spritebatch.begin thus my HUD remains stationary and does not scale.
            //I am also able to use SpriteSortMode.Immediate which is faster than BackToFront.
            //DrawOrder determines the order a component is drawn to the screen and thus it's layerdepth.
            //Lower DrawOrder is back of the screen, higher is close to your face.  Another benefit of Game Components
            //is that they all have access to Game and thus the Game Services Collection.  We can add stuff to the Game Services collection
            //like audioManager, spriteBatch and thus our game components can all share resources. 

            player = new Player(ScreenManager.Game, this, ScreenManager, playerIndex);
            player.DrawOrder = 8;
            ScreenManager.Game.Components.Add(player);

            terrain = new Terrain(ScreenManager.Game, player);  //2-3 secs
            terrain.DrawOrder = 5;
            ScreenManager.Game.Components.Add(terrain);

            frontOfPlayer = new FrontOfPlayer(ScreenManager.Game);
            frontOfPlayer.DrawOrder = 10;
            ScreenManager.Game.Components.Add(frontOfPlayer);
            #endregion

            GC.Collect();
            ScreenManager.Game.ResetElapsedTime();
        }

        //This method checks the GameScreen.IsActive property, so the game will stop 
        //updating when the pause menu is active, or if you tab away to a different application.
        public override void Update(GameTime gameTime, bool otherScreenHasFocus, bool coveredByOtherScreen) 
        {
            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            if (IsActive)
            {
                SetWorldScale();
            }
        }
        
        //Lets the game respond to player input. Unlike the Update method,
        //This will only be called when the gameplay screen is active.
        public override void HandleInput(InputState input) //Input is passed from ScreenManager.cs. 
        {                                                  //ScreenMananer instantiates the InputState Class as Input and passes it to every screen
            if (input == null)
                throw new ArgumentNullException("input");
            
            this.input = input;

            //Make sure classes where I pass playerindex are kept updated with playerindex from input class just in case it changes.
            player.playerIndex = ControllingPlayer.Value;
     
            keyboardState = input.CurrentKeyboardStates[(int)playerIndex];
            gamePadState = input.CurrentGamePadStates[(int)playerIndex];
            Vector2 leftStick = gamePadState.ThumbSticks.Left; //Make Leftstick a Vector, used in Throw Flare

            // The game pauses either if the user presses the pause button, or if
            // they unplug the active gamepad. This requires us to keep track of
            // whether a gamepad was ever plugged in, because we don't want to pause
            // on PC if they are playing with a keyboard and have no gamepad at all!
            bool gamePadDisconnected = !gamePadState.IsConnected &&
                                       input.GamePadWasConnected[(int)playerIndex];
            #region Pause Game
            if (input.IsPauseGame(ControllingPlayer) || gamePadDisconnected)
            {
                PauseGameComponentsOnOffMainMap(false);
   
                // make sure controller is not vibrating during pause
                GamePad.SetVibration((PlayerIndex)playerIndex, 0f, 0f);

                //Set ScreenManager Draworder to place PauseScreen on top of everything.  
                //Then set back to regular draworder of 4 in ScreenManager RemoveScreen Method
                ScreenManager.DrawOrder = 600;      
                ScreenManager.AddScreen(new PauseMenuScreen(), ControllingPlayer);
            }
            else
            {
                if (ComponentsMainMapPauseResume)
                {
                    PauseGameComponentsOnOffMainMap(true);
                    ComponentsMainMapPauseResume = false;
                }

                if (ComponentsCaveMapPauseResume)
                {
                    ComponentsCaveMapPauseResume = false;
                }
            }
            #endregion
        }

        public void PauseGameComponentsOnOffMainMap(bool enabled)
        {
            terrain.Enabled = enabled;
            frontOfPlayer.Enabled = enabled;
            player.Enabled = enabled;
        }

        public override void Draw(GameTime gameTime)
        {
            //Call TransfromMatrix method in camera class.  This runs the Matrix math in the draw loop.
            //I'm calling the transforMatrix method from the camera class (camera.Transform) 
            //and passing the return value into the Matrix transformMatirx overload in spritebatch.begin in the Game Component classes draw method.  
            //This turns on the camera matrix math for spriteBatch.Draw 
            //that allows scaling, rotation and panning(translation). I use a 3D camera with z set to 0.  
            ScreenManager.Camera.TransformMatrix(gameTime);
        }

        public override void UnloadContent()
        {
            ScreenManager.Game.Services.RemoveService(typeof(ContentManager));

            CleanupGameComponents(); //Remove Game Components that you have added in GamePlayScreen.cs
            content.Unload();       //This will unload the content loaded created in GamePlayScreen.cs

            base.UnloadContent();
        }

        private void CleanupGameComponents()
        {
            #region GamePlay Components
            ScreenManager.Game.Components.Remove(player);
            ScreenManager.Game.Components.Remove(terrain);
            ScreenManager.Game.Components.Remove(frontOfPlayer);
            #endregion
        }

        public void SetWorldScale() 
        {
            //Set World Scale by sending scale value to camera.  
            //Press B button or C key to Zoom in, X Button or Z key to Zoom out and A Button or X key to Reset.
            //Zoom In
            if (keyboardState.IsKeyDown(Keys.C) || gamePadState.Triggers.Right >= .2f)
            {

                if (ScreenManager.Camera.scale < ScreenManager.Camera.MinWorldScale) //Get minimum camera scale from camera class
                {
                    ScreenManager.Camera.scale += 0.010f;
                }
            }
       
            //Zoom out
            if (keyboardState.IsKeyDown(Keys.Z) || gamePadState.Triggers.Left >= .2f)
            {

                if (ScreenManager.Camera.scale > ScreenManager.Camera.MaxWorldScale) //Get maximum camera scale from camera class
                {
                    ScreenManager.Camera.scale -= 0.010f;
                }
            }
      
            //Reset Zoom
            if (keyboardState.IsKeyDown(Keys.X) || gamePadState.Buttons.A == ButtonState.Pressed)
            {
                ScreenManager.Camera.scale = 1f;
            }
        }
    }
}
