#region File Description
//-----------------------------------------------------------------------------
// MenuScreen.cs
//
// XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;

#endregion

namespace AcidRainTutorialNew
{
    /// <summary>
    /// Base class for screens that contain a menu of options. The user can
    /// move up and down to select an entry, or cancel to back out of the screen.
    /// </summary>
    abstract class MenuScreen : GameScreen
    {
        ContentManager content;
  
        List<MenuEntry> menuEntries = new List<MenuEntry>();
        int selectedEntry = 0;
        string menuTitle;
        string menuSubTitle;
 
        /// <summary>
        /// Gets the list of menu entries, so derived classes can add
        /// or change the menu contents.
        /// </summary>
        protected IList<MenuEntry> MenuEntries
        {
            get { return menuEntries; }
        }

        #region Draw Variables
        SpriteBatch spriteBatch;
        SpriteFont menuFont;
        Viewport viewport;
        Vector2 viewportSize;
        Vector2 position;
        float transitionOffset;
        Color color;
        float menuEntriesScale;
        double time;
        float pulsate;
        Vector2 origin;
        Vector2 size;
        Vector2 titlePosition; 
        #endregion

        /// <summary>
        /// Constructor.
        /// </summary>
        public MenuScreen(string menuTitle, string menuSubTitle)
        {
            this.menuTitle = menuTitle;
            this.menuSubTitle = menuSubTitle;

            TransitionOnTime = TimeSpan.FromSeconds(1.0);
            TransitionOffTime = TimeSpan.FromSeconds(1.0);
        }

        public override void LoadContent()
        {
            if (content == null)
                content = new ContentManager(ScreenManager.Game.Services, "Content");

            #region Draw Variables
            spriteBatch = ScreenManager.SpriteBatch;
            menuFont = ScreenManager.MenuFont;
            viewport = ScreenManager.GraphicsDevice.Viewport;
            viewportSize = new Vector2(viewport.Width, viewport.Height);
            #endregion

            base.LoadContent();
        }
        #region Handle Input
        /// <summary>
        /// Responds to user input, changing the selected entry and accepting
        /// or cancelling the menu.
        /// </summary>
        public override void HandleInput(InputState input)
        {
            // Move to the previous menu entry?
            if (input.IsMenuUp(ControllingPlayer))
            {
                selectedEntry--;

                if (selectedEntry < 0)
                    selectedEntry = menuEntries.Count - 1;
            }

            // Move to the next menu entry?
            if (input.IsMenuDown(ControllingPlayer))
            {
                selectedEntry++;

                if (selectedEntry >= menuEntries.Count)
                    selectedEntry = 0;
            }

            // Accept or cancel the menu? We pass in our ControllingPlayer, which may
            // either be null (to accept input from any player) or a specific index.
            // If we pass a null controlling player, the InputState helper returns to
            // us which player actually provided the input. We pass that through to
            // OnSelectEntry and OnCancel, so they can tell which player triggered them.
            PlayerIndex playerIndex;

            if (input.IsMenuSelect(ControllingPlayer, out playerIndex))
            {
                OnSelectEntry(selectedEntry, playerIndex);
            }
            else if (input.IsMenuCancel(ControllingPlayer, out playerIndex))
            {
                OnCancel(playerIndex);
            }
        }


        /// <summary>
        /// Handler for when the user has chosen a menu entry.
        /// </summary>
        protected virtual void OnSelectEntry(int entryIndex, PlayerIndex playerIndex)
        {
            menuEntries[selectedEntry].OnSelectEntry(playerIndex);
        }


        /// <summary>
        /// Handler for when the user has cancelled the menu.
        /// </summary>
        protected virtual void OnCancel(PlayerIndex playerIndex)
        {
            ExitScreen();
        }


        /// <summary>
        /// Helper overload makes it easy to use OnCancel as a MenuEntry event handler.
        /// </summary>
        protected void OnCancel(object sender, PlayerIndexEventArgs e)
        {
            OnCancel(e.PlayerIndex);
        }


        #endregion
        /// <summary>
        /// Updates the menu.
        /// </summary>
        public override void Update(GameTime gameTime, bool otherScreenHasFocus,
                                                       bool coveredByOtherScreen)
        {
            base.Update(gameTime, otherScreenHasFocus, coveredByOtherScreen);

            float elapsed = (float)gameTime.ElapsedGameTime.TotalSeconds;

            // Update each nested MenuEntry object.
            for (int i = 0; i < menuEntries.Count; i++)
            {
          
                bool isSelected = IsActive && (i == selectedEntry);

                menuEntries[i].Update(this, isSelected, gameTime);
            }
        }

        
        /// <summary>
        /// Draws the menu.
        /// </summary>
        public override void Draw(GameTime gameTime)
        {
            position = new Vector2(viewport.X / 2, viewportSize.Y * 0.55f);

            // Make the menu slide into place during transitions, using a
            // power curve to make things look more interesting (this makes
            // the movement slow down as it nears the end).
            transitionOffset = (float)Math.Pow(TransitionPosition, 2);
            
            if (ScreenState == ScreenState.TransitionOn)
                position.Y += transitionOffset * 256;
            else
                position.Y += transitionOffset * 512;

            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None);

            #region Draw Menu Entries
            // Draw each menu entry in turn.
            for (int i = 0; i < menuEntries.Count; i++)
            {
                color = ScreenManager.Color;
   
                menuEntriesScale = .9f;

                if (IsActive && (i == selectedEntry))
                {
                    // The selected entry is Orange, and has an animating size.
                    time = gameTime.TotalGameTime.TotalSeconds;
                    pulsate = (float)Math.Sin(time * 6f) + 1f;

                    color = Color.Orange;
                    menuEntriesScale += pulsate * 0.05f;
                }

                // Modify the alpha to fade text out during transitions.
                color = new Color(color.R, color.G, color.B, TransitionAlpha);

                // Draw text, centered on the middle of each line.
                origin = new Vector2(0, menuFont.LineSpacing / 2);
                size = menuFont.MeasureString(menuEntries[i].Text);
                position.X = viewportSize.X / 2f - size.X / 2f * menuEntriesScale;

                //Draw Text Shadow
                spriteBatch.DrawString(menuFont, menuEntries[i].Text, new Vector2(position.X + 1, position.Y + 1), 
                                       new Color(99, 99, 99, TransitionAlpha), 0, origin, menuEntriesScale, SpriteEffects.None, 0);
             
                //Draw Text
            
                spriteBatch.DrawString(menuFont, menuEntries[i].Text, position, color, 0, origin, menuEntriesScale, SpriteEffects.None, 0);

                position.Y += menuFont.LineSpacing - 10;
            }
            #endregion

            #region Set position of Title and SubTitle
            titlePosition = new Vector2(viewportSize.X / 2, viewportSize.Y * 0.11f);
            titlePosition.Y -= (float)Math.Pow(TransitionPosition, 2) * titlePosition.Y;
      
            #endregion

            spriteBatch.End();
        }
    }
}
